angular.module('koClassified.controllers.addAds', [])

.controller('addAdsController', ['$scope', '$rootScope', '$ionicPopup', '$ionicActionSheet', '$cordovaFileTransfer', '$cordovaToast', '$filter', 'navigationService', 'modalService', 'adsService', 'imageService', 'adsFactory', 'authService', 'paymentService', 'lodash',
                        function( $scope,   $rootScope,   $ionicPopup,   $ionicActionSheet,   $cordovaFileTransfer,   $cordovaToast,   $filter,   navigationService,   modalService,   adsService,   imageService,   adsFactory,   authService,   paymentService,  _)
{
    $scope.$on('$ionicView.beforeEnter', function() {

        $scope.imagesCount = 1;
        $scope.hasLink = false;
        $scope.hasTags = false;
        $scope.canAddSpecialItem = true;
        $scope.setImagesArray();

        $scope.adsService = adsService;

        adsService.loadDefaultPlan().then(function(default_plan) {
            $scope.imagesCount = default_plan.MAX_IMG_COUNT;
            $scope.planChanged(default_plan);
        });

        authService.checkTokenExpiry();
        adsService.loadSettings().then(function(settings) {
            if(settings.APP_HAS_TAGS === 'YES') {
                $scope.hasTags = true;
            }
            if(settings.APP_CAN_ADD_SPECIAL_ADS !== 'YES') {
                $scope.canAddSpecialItem = false;
            }
            if(settings.SUBMIT__ENABLE_FULL_ADDRESS_FIELD === 'YES') {
                $scope.hasAddress = true;
            }
        });

        $scope.submitted = false;
        $scope.modals = modalService.init($scope);
        $scope.currentCategory = $scope.currentSubCategory = $scope.currentSubsidiary = 0;
        $scope.custom_fields = null;
        $scope.customFieldData = {};

        //$scope.tags = [
        //    { text: 'Tag1' },
        //    { text: 'Tag3' }
        //];
    });
    $scope.submit_button_text = 'ثبت آگهی';

    $scope.radioButtonChanged = function(field_name, key) {
        $scope.customFieldData[field_name] = key;
    };


    $rootScope.$on("submitAdsForm", function(){
        $scope.submitForm($scope.vm.addForm);
        // $scope.parentmethod();
    });

   // because we want to submit the form in footer scope we should use dot notation for form name
    $scope.vm = {
        addForm : null
    };
    $scope.authService = authService;

    $scope.formData = {};
    $scope.saved = false;
    $scope.currentImage = null;
    $scope.promises = {};

    $scope.priceTypes = {
        free : 'رایگان',
        deal : 'توافقی',
        exchange : 'جهت معاوضه'
    };

    $scope.showImage = function(image) {
        if(! image.data) {
            return 'img/placeholder.png';
        }

        return image.data;
        // return 'data:image/jpeg;base64,' + image.data;
    };

    $scope.imageClicked = function(image) {
        if(! image.data) {
            return $scope.chooseImageType(image);
        }

        if(image.isLoading) {
            return false;
        }

        return $scope.imageActionSheet(image);
    };

    $scope.deleteImage = function(image) {
        image.uploaded_name = null;
        image.data          = null;
        image.isLoading     = false;
        image.isUploaded    = false;
        image.isFailed      = false;
    };

    $scope.imageActionSheet = function(image) {
        $ionicActionSheet.show({
            buttons: [
                { text: '<i class="icon ion-images"></i> تغییر این تصویر' },
                { text: '<i class="icon ion-trash-b"></i> حذف' },
                { text: '<i class="icon ion-ios-close-empty"></i> بی خیال!' }
            ],

            buttonClicked: function(index) {
                if(index === 0) {
                    $scope.chooseImageType(image);
                    return true;
                }
                if(index == 1) {
                    $scope.deleteImage(image);
                    return true;
                }
                if(index == 2) {
                    return true;
                }
            }
        });
    };

    /* popup to show options for picking images */
    $scope.chooseImageType = function(image) {
      $scope.currentImage = image;
      $scope.chooseImageTypePopup = $ionicPopup.show({
        templateUrl : 'templates/popups/choose_image_type.html',
        scope : $scope,
        cssClass : 'no-header-popup loading-popup ' + $scope.main_theme + ''

      });
    };

    $scope.upload = function(imageData, currentImage) {

        adsFactory.upload(imageData).then(function(result) {

            var response = JSON.parse(result.response);
            if(response.status) {
                currentImage.uploaded_name = response.image_name;
                currentImage.isLoading = false;
                currentImage.isUploaded = true;
            } else {
                currentImage.isLoading = false;
                currentImage.isFailed = true;
                $cordovaToast.showShortBottom(response.errors);
            }
        }, function(err) {
            currentImage.isLoading = false;
            currentImage.isFailed = true;
            $cordovaToast.showShortBottom('مشکلی در آپلود تصویر رخ داده است');
            // console.log("ERROR for " + currentImage.id + " : " + JSON.stringify(err));
        }, function (progress) {
             // console.log('PROGRESS FOR ' + currentImage.id + ' = ' +(progress.loaded / progress.total) * 100);
        });
    };

  /* handles adding the image to form data */
    $scope.takePicture = function(type) {
      $scope.chooseImageTypePopup.close();

      imageService.takePicture(type).then(function(imageData) {
          $scope.currentImage.isFailed = false;
          $scope.currentImage.isUploaded = false;
          $scope.currentImage.isLoading = true;

          $scope.currentImage.data = imageData + '?' + Math.random();
          $scope.upload(imageData, $scope.currentImage);
      }, function(err) {
          console.log('err in controller');
      });
    };

    /*
     * Save the state id on selection
     */
    $scope.citiesModalOnShow = function() {
        $scope.formData.state = modalService.currentSelection;
    };

    /*
    * Submit the form
    */
    $scope.submitForm = function(form) {
        $scope.form = form;
        $scope.submitted = true;

        if($scope.isLoadingAds) {
            return false;
        }

        if(form.$invalid) {
            $cordovaToast.showShortBottom('اطلاعات وارد شده صحیح نمی باشد');
            return false;
        }

        if(_.find($scope.formData.images, {isLoading: true})) {
            $cordovaToast.showShortBottom('لطفا پس از اتمام آپلود تصاویر اقدام به ثبت نمایید');
            return false;
        }

        $scope.saved = false;
        if( ! authService.isLoggedIn() ) {
            authService.authType = 'submit';
            navigationService.go('app.auth');
            return false;
        }

        $scope.loading_text = 'در حال ارسال اطلاعات';
        $scope.loadingPopup = $ionicPopup.show({
            templateUrl: 'templates/popups/adding_ads.html',
            cssClass: 'no-header-popup loading-popup',
            scope: $scope
        });

        /*
         * INTERNAL :: returns the real price to be send into database
         */
        var priceToSend = function() {
          if($scope.formData.price.checkbox) {
              return 'DEAL';
          }

          return $scope.formData.price.input;
        };

        var owner_phone = null;
        var owner_email = null;
        if($scope.formData.phone) {
             owner_phone = '0'+$scope.formData.phone;
        }
        if($scope.formData.owner_email) {
            owner_email = $scope.formData.owner_email;
        }

        var data = {
            title :         $scope.formData.title,
            category_id :   $scope.formData.category.ID,
            custom_field_categories: {
                category:    $scope.currentCategory,
                subcategory: $scope.currentSubCategory,
                subsidiary:  $scope.currentSubsidiary
            },
            state_id:       $scope.formData.state.ID,
            location_id :   $scope.formData.location.ID,
            description :   $scope.formData.description,
            owner_phone :   owner_phone,
            owner_email :   owner_email,
            images :        _.map($scope.formData.images, 'uploaded_name'),
            tags:           $scope.formData.tags,
            link:           $scope.formData.link,
            address:        $scope.formData.address
            //price :       priceToSend(),
        };

        if($scope.canAddSpecialItem) {
            data.plan = $scope.formData.plan.ID;
        } else {
            data.plan = adsService.defaultPlan.ID;
        }

        var custom_field_data_to_be_sent = [];
        var value = null;

        angular.forEach($scope.custom_fields, function(field)
        {
            value = $scope.customFieldData[field.field_name];
            if(field.type == 'price-popup') {
                if(! value.checkbox) {
                    value = value.input;
                } else {
                    value = value.checkbox;
                }
            } else if(field.type == 'select') {
                value = value.value;
            }

            custom_field_data_to_be_sent.push({
                field_name: field.field_name,
                field_id: field.id,
                value: value
            });
        });

        data.custom_fields = custom_field_data_to_be_sent;
        data.user_token = authService.token;

        adsService.post(data).then(function(result)
        {
            if(result.status === 'success') {
                $scope.saved = true;
                $scope.created_ads = result.ads;
            }

            if(result.status === 'success' && result.needs_to_pay) { //payment
                $scope.loading_text = 'در حال اتصال به درگاه پرداخت';
                paymentService.setScope($scope);
                paymentService.start($scope.created_ads.ID, $scope.created_ads.special.ID, result.user_id, result.payment_price, 'add-new', 0).then(
                    function() {
                        // success
                    }, function() {
                        $scope.loadingPopup.close();
                    }
                );
            } else if(result.status == 'success' && !result.needs_to_pay) { // free ads
                $scope.loadingPopup.close();
                var addedPopup = $ionicPopup.show({
                    templateUrl: 'templates/popups/added_ads.html',
                    cssClass: 'no-header-popup loading-popup',
                    scope: $scope,
                    buttons: [
                      {
                        text: '<b>متوجه شدم</b>',
                        type: 'button-energized',
                        onTap: function() {
                            addedPopup.close();
                            $scope.goBack();
                        }
                      }
                    ]
                });

                $scope.closeForm();

            }else {
                if(result.errors.auth) {
                    $scope.loadingPopup.close();
                    $cordovaToast.showShortBottom(result.errors.auth);
                }
            }
        },function() {
            $scope.loadingPopup.close();
        });
    };

    $scope.paymentSuccessCallback = function() {
        $scope.payment_data = paymentService.payment_data;
        $scope.loadingPopup.close();

        var successPaymentPopup = $ionicPopup.show({
            templateUrl: 'templates/popups/success_payment_popup.html',
            cssClass: 'no-header-popup loading-popup success-payment-popup',
            scope: $scope,
            buttons: [{
                text: '<b>متوجه شدم</b>',
                type: 'button-energized',
                onTap: function() {
                    successPaymentPopup.close();
                    $scope.goBack();
                }
            }]
        });

        $scope.closeForm();
    };

    $scope.paymentErrorCallback = function() {
        $scope.payment_data = paymentService.payment_data;
        $scope.loadingPopup.close();
        $scope.is_creating = true;
        var failedPaymentPopup = $ionicPopup.show({
            templateUrl: 'templates/popups/failed_payment_popup.html',
            cssClass: 'no-header-popup loading-popup failed-payment-popup',
            scope: $scope,
            buttons: [{
                text: '<b>متوجه شدم</b>',
                type: 'button-energized',
                onTap: function() {
                    failedPaymentPopup.close();
                    $scope.goBack();
                }
            }]
        });

        $scope.closeForm();
    };

    /*
     * resets the add ads form
     */
    $scope.closeForm = function(popup) {
        $scope.formData = {};
        $scope.form.$setPristine();
        $scope.imageData = null;
        $scope.saved = false;
        $scope.submitted = false;

        if(popup) {
            popup.close();
        }
    };

    /*
    * Shows the popup for price selection
    */
    $scope.showPricePopup = function(popup_name) {
        var cssClass = 'no-header-popup loading-popup ' + $scope.main_theme + ''; //main_theme comes from rootScope
        $scope.current_popup_field = popup_name;
        popup_name = popup_name + 'Popup';

        $scope[popup_name] = $ionicPopup.show({
            templateUrl : 'templates/popups/add_price.html',
            scope : $scope,
            cssClass : cssClass,
            buttons: [
                {
                    text: '<b> اعمال قیمت</b>',
                    type: 'button-' + $scope.secondary_theme, // secondary_theme comes from rootScope
                    onTap: function() {
                        applyPrice(popup_name, $scope.current_popup_field);
                    }
                }
            ]
        });
    };

    $scope.numberFormat = function() {
        if(!$scope.customFieldData[$scope.current_popup_field].input || $scope.customFieldData[$scope.current_popup_field].input === 0 || $scope.customFieldData[$scope.current_popup_field].input === ''){
          $scope.customFieldData[$scope.current_popup_field].text = '';
          return false;
        }
        $scope.customFieldData[$scope.current_popup_field].checkbox = false;
        $scope.customFieldData[$scope.current_popup_field].text = $filter('currency')($scope.customFieldData[$scope.current_popup_field].input, '', 0) + ' تومان';
        return false;
    };

    /*
     * Function to be called when checkboxes in price popup changed
     */
    $scope.pricePopupCheckboxChanged = function(value) {
        if(value) {
            $scope.customFieldData[$scope.current_popup_field].input = '';
            $scope.customFieldData[$scope.current_popup_field].text = $scope.priceTypes[value];
        } else {
            $scope.customFieldData[$scope.current_popup_field].text = null;
        }
    };

    /*
    * functions to be called when applying price on popup
    */
    var applyPrice = function(popup_name, field_name) {
        if(!$scope.customFieldData[field_name].checkbox) {
          if($scope.customFieldData[field_name].input) {
            $scope.customFieldData[field_name].text = $filter('currency')($scope.customFieldData[field_name].input, '', 0) + ' تومان';
          } else {
            $scope.customFieldData[field_name].checkbox = 'free';
            $scope.customFieldData[$scope.current_popup_field].text = $scope.priceTypes.free;
          }
        }

        $scope[popup_name].close();
    };

    /* Initiate Modal Service */
    $scope.modals = modalService.init($scope);

    /*-- set a location --*/
    $scope.setLocation = function(location) {
        $scope.formData.location = location;
    };

    $scope.setSpecificCategory = function(category) {
        $scope.currentCategory = category.ID;
        $scope.currentSubCategory = $scope.currentSubsidiary = 0;
    };
    $scope.setSpecificSubCategory = function(subcategory) {
        $scope.currentSubCategory = subcategory.ID;
        $scope.currentSubsidiary = 0;
    };
    $scope.setSpecificSubCategorySubsidiary = function(subsidiary) {
        $scope.currentSubsidiary = subsidiary.ID;
    };

    /*-- set a category --*/
    $scope.setCategory = function(category) {
        $scope.formData.category = category;
        if(category.PARENT !== 0) {
            $scope.setSpecificSubCategorySubsidiary(category);
        }
        $scope.is_loading_custom_fields = true;
        adsService.loadCustomFields($scope.currentCategory, $scope.currentSubCategory, $scope.currentSubsidiary).then(function(custom_fields) {

            $scope.is_loading_custom_fields = false;
            $scope.custom_fields = custom_fields;
            var selected_option = null;
            angular.forEach(adsService.getRadioBoxes(custom_fields), function(field) {
                selected_option = _.find(field.options, {selected: 1});
                $scope.customFieldData[field.field_name] = selected_option.option_key;
            });
        });
    };

    /*-- set a plan --*/
    $scope.setPlan = function(plan) {
        $scope.formData.plan = plan;
        $scope.planChanged(plan);
        if(adsService.isSpecialPlan(plan)) {
            $scope.submit_button_text = 'ثبت آگهی و پرداخت';
        }else{
            $scope.submit_button_text = 'ثبت آگهی';
        }
    };

    $scope.setCustomFieldModalData = function(field, key, text) {
        $scope.customFieldData[field.field_name] = {
            value: key,
            text: text
        };
    };
    /*-- Reset Location --*/
    $scope.resetLocation = function() {
      $scope.location = {};
      $scope.adss.load();
    };
    /*-- Reset Category --*/
    $scope.resetCategory = function() {
      $scope.category = {};
      $scope.adss.load();
    };

    /*
     * function that handles the back button
     */
    $scope.backButtonClicked = function() {
      var form = $scope.vm.addForm;

      if(form.$pristine || $scope.saved) {
          return navigationService.go('app.list');
      }

      var exitPopup = $ionicPopup.show({
        template: 'تغییرات شما هنوز ذخیره  نشده است. آیا مایلید که از این صفحه خارج شوید؟',
        cssClass: 'no-header-popup loading-popup',
        scope: $scope,
        buttons: [
          {
            text: '<b>بلی</b>',
            type: 'button-balanced',
            onTap: function() {
              exitPage();
            }
          },
          {
            text: '<b>خیر</b>',
            type: 'button-assertive',
            onTap: function() {
              exitPopup.close();
            }
          }
        ]
      });

      /*
       * internal Function for exiting the add ads page
       */
      var exitPage = function() {
          $scope.formData = {};
          $scope.imageData = null;
          $scope.saved = false;
          form.$setPristine();
          $scope.submitted = false;
          exitPopup.close();
          navigationService.go('app.list');
      };
    };

    /*
     * Redirects back
     */
    $scope.goBack = function() {
      navigationService.go('app.list');
    };

    /**
     * Resets the images Array
     */
    $scope.setImagesArray = function() {
        if(typeof $scope.formData.images == 'undefined') {
            $scope.formData.images = [];
        }

        var start = 1;
        var end   = $scope.imagesCount;
        if($scope.formData.images.length > 0) {
            start = $scope.formData.images.length + 1;
        }

        // if should delete some images
        if($scope.imagesCount < $scope.formData.images.length) {
            start = $scope.imagesCount + 1;
            end = $scope.formData.images.length;
            for(var i = start; i <= end; i++) {
                _.remove($scope.formData.images, {'id': i});
            }

            return false;
        }

        // if should add some images
        for(var j = start; j <= end; j++) {
            $scope.formData.images.push({
                id: j,
                isLoading: false,
                isUploaded: false,
                isFailed: false,
                data: null,
                uploaded_name: null
            });
        }
    };

    /**
     * Callback for when plan is changed
     * @param new_plan
     */
    $scope.planChanged = function(new_plan) {
        $scope.imagesCount = new_plan.MAX_IMG_COUNT;
        $scope.hasLink = false;
        if(new_plan.HAS_LINK == 2) {
            $scope.hasLink = true;
        }
        $scope.setImagesArray();
    };

    $scope.expandText = function(input_id){
        var element = document.getElementById(input_id);
        var currentHeight = element.scrollHeight;
        if(currentHeight >= 100) {
            element.style.height = currentHeight + "px";
        }
    };
}]);
