angular.module('koClassified.controllers.editAds', [])

.controller('editAdsController', ['$scope', '$stateParams', '$ionicPopup', '$ionicActionSheet', '$cordovaFileTransfer', '$cordovaToast', '$filter', 'navigationService', 'modalService', 'adsService', 'imageService', 'adsFactory', 'authService', 'paymentService', 'lodash',
                         function( $scope,    $stateParams,   $ionicPopup,   $ionicActionSheet,   $cordovaFileTransfer,   $cordovaToast,   $filter,   navigationService,   modalService,   adsService,   imageService,   adsFactory,   authService,   paymentService,  _)
{
    $scope.$on('$ionicView.beforeEnter', function() {

        $scope.isLoadingAds = true;
        $scope.imagesCount = 1;
        $scope.hasLink = false;
        $scope.hasTags = false;

        $scope.adsService = adsService;
        $scope.random_number = Math.random();

        adsService.loadSettings().then(function(settings) {
            if(settings.APP_HAS_TAGS === 'YES') {
                $scope.hasTags = true;
            }
            if(settings.SUBMIT__ENABLE_FULL_ADDRESS_FIELD === 'YES') {
                $scope.hasAddress = true;
            }
        });

        $scope.submitted = false;
        $scope.modals = modalService.init($scope);
        $scope.currentCategory = $scope.currentSubCategory = $scope.currentSubsidiary = 0;
        $scope.custom_fields = null;
        $scope.customFieldData = {};

        $scope.tags = [
            { text: 'Tag1' },
            { text: 'Tag2' },
            { text: 'Tag3' }
        ];

        $scope.radioButtonChanged = function(field_name, key) {
            $scope.customFieldData[field_name] = key;
        }

        var adsId = $stateParams.id;
        adsService.findWithIdForEdit(adsId).then(function(response) {

            $scope.ads = response.ads;
            $scope.images = adsService.images($scope.ads.image);
            $scope.custom_fields = response.custom_fields;
            $scope.custom_values = response.custom_values;

            $scope.imagesCount = $scope.ads.special.plan.MAX_IMG_COUNT;
            if($scope.ads.special.plan.HAS_LINK == 2) {
                $scope.hasLink = true;
            }

            $scope.formData = {
                title:      $scope.ads.ADS_TITLE,
                description: $scope.ads.safe_description,
                location:   $scope.ads.city,
                state:      $scope.ads.state,
                category:   $scope.ads.branch,
                plan:       $scope.ads.special.plan,
                price: {
                    input:  $scope.ads.ADS_PRICE,
                    text:   $scope.ads.ADS_PRICE,
                    checkbox: false
                },
                phone:       parseFloat($scope.ads.ADS_OWNER_PHONE),
                owner_email: $scope.ads.ADS_OWNER_EMAIL,
                link:        $scope.ads.ADS_LINK,
                address:     $scope.ads.ADS_OWNER_ADDRESS
            };

            $scope.currentCategory    = $scope.ads.branch.category.ID;
            $scope.currentSubCategory = $scope.ads.branch.ID;
            $scope.currentSubsidiary  = 0;

            if($scope.ads.branch.parent) {
                $scope.currentSubCategory = $scope.ads.branch.parent.ID;
                $scope.currentSubsidiary  = $scope.ads.branch.ID;
            }

            // setting current custom field values
            angular.forEach($scope.custom_fields, function(custom_field) {
                var field = _.find($scope.custom_values, { field_id: custom_field.id});
                var value = field.info_value;

                if(custom_field.type == 'number') {
                    value = parseInt(value);
                }
                if(custom_field.type != 'price-popup') {
                    $scope.customFieldData[custom_field.field_name] = value;
                }

                if(custom_field.type == 'select') {
                    $scope.customFieldData[custom_field.field_name] = {
                        value: value,
                        text: field.text_value
                    };
                }

                if(custom_field.type == 'price-popup') {
                    if(value == 'deal' || value == 'exchange' || value == 'free') {
                        $scope.customFieldData[custom_field.field_name] = {
                            input:  value,
                            text:   field.text_value,
                            checkbox: true
                        };
                    } else {
                        $scope.customFieldData[custom_field.field_name] = {
                            input:  value,
                            text:   $filter('currency')(value, '', 0) + ' تومان',
                            checkbox: false
                        };
                    }
                }

            });

            $scope.setImagesArray();

            if($scope.ads.ADS_PRICE == 'DEAL' || ! $scope.ads.ADS_PRICE) {
                $scope.formData.price.checkbox = true;
                $scope.formData.price.text = ' توافقی';
                $scope.formData.price.input = 'DEAL';
            } else if($scope.ads.ADS_PRICE) {
                $scope.formData.price.text = $filter('currency')($scope.ads.ADS_PRICE, '', 0) + ' تومان';
            }

            $scope.isLoadingAds = false;
        });


    });
    $scope.submit_button_text = 'ویرایش آگهی';

   // because we want to submit the form in footer scope we should use dot notation for form name
    $scope.vm = {
      addForm : null
    };
    $scope.authService = authService;

    $scope.formData = {};
    $scope.formData.email = authService.getEmail();
    $scope.saved = false;
    $scope.currentImage = null;
    $scope.promises = {};

    $scope.priceTypes = {
        free : 'رایگان',
        deal : 'توافقی',
        exchange : 'جهت معاوضه'
    };

    $scope.showImage = function(image) {
        if(! image.data) {
            return 'img/placeholder.png';
        }

        return image.data +'?'+ $scope.random_number;
    };

    $scope.imageClicked = function(image) {
        if(! image.data) {
            return $scope.chooseImageType(image);
        }

        if(image.isLoading) {
            return false;
        }

        return $scope.imageActionSheet(image);
    };

    $scope.deleteImage = function(image) {
        image.uploaded_name = null;
        image.data          = null;
        image.isLoading     = false;
        image.isUploaded    = false;
        image.isFailed      = false;
        image.isPreUploaded = false;
    };

    $scope.imageActionSheet = function(image) {
        $ionicActionSheet.show({
            buttons: [
                { text: '<i class="icon ion-images"></i> تغییر این تصویر' },
                { text: '<i class="icon ion-trash-b"></i> حذف' },
                { text: '<i class="icon ion-ios-close-empty"></i> بی خیال!' }
            ],

            buttonClicked: function(index) {
                if(index === 0) {
                    $scope.chooseImageType(image);
                    return true;
                }
                if(index == 1) {
                    $scope.deleteImage(image);
                    return true;
                }
                if(index == 2) {
                    return true;
                }
            }
        });
    };

    /* popup to show options for picking images */
    $scope.chooseImageType = function(image) {
      $scope.currentImage = image;
      $scope.chooseImageTypePopup = $ionicPopup.show({
        templateUrl : 'templates/popups/choose_image_type.html',
        scope : $scope,
        cssClass : 'no-header-popup loading-popup ' + $scope.main_theme + ''

      });
    };

    $scope.upload = function(imageData, currentImage) {

        adsFactory.upload(imageData).then(function(result) {

            var response = JSON.parse(result.response);
            if(response.status) {
                currentImage.uploaded_name = response.image_name;
                currentImage.isLoading = false;
                currentImage.isUploaded = true;
            } else {
                currentImage.isLoading = false;
                currentImage.isFailed = true;
                $cordovaToast.showShortBottom(response.errors);
            }


        }, function(err) {
            currentImage.isLoading = false;
            currentImage.isFailed = true;
            $cordovaToast.showShortBottom('مشکلی در آپلود تصویر رخ داده است');
            // console.log("ERROR for " + currentImage.id + " : " + JSON.stringify(err));
        }, function (progress) {
             // console.log('PROGRESS FOR ' + currentImage.id + ' = ' +(progress.loaded / progress.total) * 100);
        });
    };

  /* handles adding the image to form data */
    $scope.takePicture = function(type) {
      $scope.chooseImageTypePopup.close();

      imageService.takePicture(type).then(function(imageData) {
          $scope.currentImage.isFailed = false;
          $scope.currentImage.isUploaded = false;
          $scope.currentImage.isLoading = true;

          $scope.currentImage.data = imageData + '?' + Math.random();
          $scope.upload(imageData, $scope.currentImage);
      }, function(err) {
          console.log('err in controller');
      });
    };

    /*
     * Save the state id on selection
     */
    $scope.citiesModalOnShow = function() {
        $scope.formData.state = modalService.currentSelection;
    };

    $scope.submitClicked = function(form) {
        $scope.form = form;
        $scope.submitted = true;

        if(form.$invalid) {
            $cordovaToast.showShortBottom('اطلاعات وارد شده صحیح نمی باشد');
            return false;
        }

        if(_.find($scope.formData.images, {isLoading: true})) {
            $cordovaToast.showShortBottom('لطفا تا زمان اتمام آپلود تصاویر منتظر بمانید');
            return false;
        }

        $scope.showConfirmEditPopup();
    };

    $scope.showConfirmEditPopup = function() {
        var cssClass = 'no-header-popup loading-popup confirm-popup ' + $scope.main_theme + ''; //main_theme comes from rootScope
        $scope.confirmEditPopup = $ionicPopup.show({
            templateUrl : 'templates/popups/confirm_edit.html',
            scope : $scope,
            cssClass : cssClass
        });
    };

    $scope.closeConfirmEditPopup = function() {
      $scope.confirmEditPopup.close();
    };

    /*
    * Submit the form
    */
    $scope.submitForm = function() {
        $scope.closeConfirmEditPopup();
        $scope.saved = false;

        $scope.loading_text = 'در حال ارسال اطلاعات';
        $scope.loadingPopup = $ionicPopup.show({
            templateUrl: 'templates/popups/adding_ads.html',
            cssClass: 'no-header-popup loading-popup',
            scope: $scope
        });

        /*
         * INTERNAL :: returns the real price to be send into database
         */
        var priceToSend = function() {
          if($scope.formData.price.checkbox) {
              return 'DEAL';
          }

          return $scope.formData.price.input;
        };

        var owner_phone = null;
        var owner_email = null;
        if($scope.formData.phone) {
            owner_phone = '0'+$scope.formData.phone;
        }
        if($scope.formData.owner_email) {
            owner_email = $scope.formData.owner_email;
        }

        var data = {
            ads_id:       $scope.ads.ID,
            title :       $scope.formData.title,
            category_id : $scope.formData.category.ID,
            custom_field_categories: {
                category: $scope.currentCategory,
                subcategory: $scope.currentSubCategory,
                subsidiary:  $scope.currentSubsidiary
            },
            state_id:     $scope.formData.state.ID,
            location_id : $scope.formData.location.ID,
            description : $scope.formData.description,
            owner_phone :   owner_phone,
            owner_email :   owner_email,
            price :       priceToSend(),
            images :      $scope.formData.images,
            tags:         $scope.formData.tags,
            link:         $scope.formData.link,
            address:      $scope.formData.address
        };


        var custom_field_data_to_be_sent = [];
        var value = null;

        angular.forEach($scope.custom_fields, function(field)
        {
            value = $scope.customFieldData[field.field_name];
            if(field.type == 'price-popup') {
                if(! value.checkbox) {
                    value = value.input;
                } else {
                    value = value.checkbox;
                }
            } else if(field.type == 'select') {
                value = value.value;
            }

            custom_field_data_to_be_sent.push({
                field_name: field.field_name,
                field_id: field.id,
                value: value
            });
        });

        data.custom_fields = custom_field_data_to_be_sent;
        console.log(data);

        if(authService.isLoggedIn()) {
            data.user_token = authService.token;
        } else {
            data.email = $scope.formData.email;
            data.password = $scope.formData.password;
            authService.saveEmail($scope.formData.email);
        }

        adsService.update(data).then(function(result)
        {
            if(result.status === 'success') {
                $scope.saved = true;
                $scope.needs_to_be_approved = result.needs_to_be_approved;
                $scope.loadingPopup.close();

                var addedPopup = $ionicPopup.show({
                    templateUrl: 'templates/popups/updated_ads.html',
                    cssClass: 'no-header-popup loading-popup',
                    scope: $scope,
                    buttons: [
                      {
                        text: '<b>متوجه شدم</b>',
                        type: 'button-energized',
                        onTap: function() {
                          addedPopup.close();
                          $scope.goBack();
                        }
                      }
                    ]
                });

                $scope.closeForm();

            }else {
              if(result.errors.auth) {
                  $scope.loadingPopup.close();
                  $cordovaToast.showShortBottom(result.errors.auth);
              }
            }
        },function() {
            $scope.loadingPopup.close();
        });
    };

    $scope.paymentSuccessCallback = function() {
        $scope.payment_data = paymentService.payment_data;
        $scope.loadingPopup.close();

        var successPaymentPopup = $ionicPopup.show({
            templateUrl: 'templates/popups/success_payment_popup.html',
            cssClass: 'no-header-popup loading-popup success-payment-popup',
            scope: $scope,
            buttons: [{
                text: '<b>متوجه شدم</b>',
                type: 'button-energized',
                onTap: function() {
                    successPaymentPopup.close();
                    $scope.goBack();
                }
            }]
        });

        $scope.closeForm();
    };

    $scope.paymentErrorCallback = function() {
        $scope.payment_data = paymentService.payment_data;
        $scope.loadingPopup.close();
        $scope.is_creating = true;
        var failedPaymentPopup = $ionicPopup.show({
            templateUrl: 'templates/popups/failed_payment_popup.html',
            cssClass: 'no-header-popup loading-popup failed-payment-popup',
            scope: $scope,
            buttons: [{
                text: '<b>متوجه شدم</b>',
                type: 'button-energized',
                onTap: function() {
                    failedPaymentPopup.close();
                    $scope.goBack();
                }
            }]
        });

        $scope.closeForm();
    };

    /*
     * resets the add ads form
     */
    $scope.closeForm = function(popup) {
        $scope.formData = {};
        $scope.form.$setPristine();
        $scope.imageData = null;
        $scope.saved = false;
        $scope.submitted = false;

        if(popup) {
            popup.close();
        }
    };

    /*
    * Shows the popup for price selection
    */
    $scope.showPricePopup = function(popup_name) {
        var cssClass = 'no-header-popup loading-popup ' + $scope.main_theme + ''; //main_theme comes from rootScope
        $scope.current_popup_field = popup_name;
        popup_name = popup_name + 'Popup';

        $scope[popup_name] = $ionicPopup.show({
            templateUrl : 'templates/popups/add_price.html',
            scope : $scope,
            cssClass : cssClass,
            buttons: [
                {
                    text: '<b> اعمال قیمت</b>',
                    type: 'button-' + $scope.secondary_theme, // secondary_theme comes from rootScope
                    onTap: function() {
                        applyPrice(popup_name, $scope.current_popup_field);
                    }
                }
            ]
        });
    };

    $scope.numberFormat = function() {
        if(!$scope.customFieldData[$scope.current_popup_field].input || $scope.customFieldData[$scope.current_popup_field].input === 0 || $scope.customFieldData[$scope.current_popup_field].input === ''){
          $scope.customFieldData[$scope.current_popup_field].text = '';
          return false;
        }
        $scope.customFieldData[$scope.current_popup_field].checkbox = false;
        $scope.customFieldData[$scope.current_popup_field].text = $filter('currency')($scope.customFieldData[$scope.current_popup_field].input, '', 0) + ' تومان';
        return false;
    };

    /*
     * Function to be called when checkboxes in price popup changed
     */
    $scope.pricePopupCheckboxChanged = function(value) {
        if(value) {
            $scope.customFieldData[$scope.current_popup_field].input = '';
            $scope.customFieldData[$scope.current_popup_field].text = $scope.priceTypes[value];
        } else {
            $scope.customFieldData[$scope.current_popup_field].text = null;
        }
    };

    /*
    * functions to be called when applying price on popup
    */
    var applyPrice = function(popup_name, field_name) {
        if(!$scope.customFieldData[field_name].checkbox) {
          if($scope.customFieldData[field_name].input) {
            $scope.customFieldData[field_name].text = $filter('currency')($scope.customFieldData[field_name].input, '', 0) + ' تومان';
          } else {
            $scope.customFieldData[field_name].checkbox = 'free';
            $scope.customFieldData[$scope.current_popup_field].text = $scope.priceTypes.free;
          }
        }

        $scope[popup_name].close();
    };

    /* Initiate Modal Service */
    $scope.modals = modalService.init($scope);

    /*-- set a location --*/
    $scope.setLocation = function(location) {
        $scope.formData.location = location;
    };

    $scope.setSpecificCategory = function(category) {
        $scope.currentCategory = category.ID;
        $scope.currentSubCategory = $scope.currentSubsidiary = 0;
    };
    $scope.setSpecificSubCategory = function(subcategory) {
        $scope.currentSubCategory = subcategory.ID;
        $scope.currentSubsidiary = 0;
    };
    $scope.setSpecificSubCategorySubsidiary = function(subsidiary) {
        $scope.currentSubsidiary = subsidiary.ID;
    };

    /*-- set a category --*/
    $scope.setCategory = function(category) {
        $scope.formData.category = category;
        if(category.PARENT !== 0) {
            $scope.setSpecificSubCategorySubsidiary(category);
        }
        $scope.is_loading_custom_fields = true;
        adsService.loadCustomFields($scope.currentCategory, $scope.currentSubCategory, $scope.currentSubsidiary).then(function(custom_fields) {

            $scope.is_loading_custom_fields = false;
            $scope.custom_fields = custom_fields;
            var selected_option = null;
            angular.forEach(adsService.getRadioBoxes(custom_fields), function(field) {
                selected_option = _.find(field.options, {selected: 1});
                $scope.customFieldData[field.field_name] = selected_option.option_key;
            });
        });
    };

    /*-- set a plan --*/
    $scope.setPlan = function(plan) {
        $scope.formData.plan = plan;
        $scope.planChanged(plan);
        if(adsService.isSpecialPlan(plan)) {
            $scope.submit_button_text = 'ثبت آگهی و پرداخت';
        }else{
            $scope.submit_button_text = 'ثبت آگهی';
        }
    };

    $scope.setCustomFieldModalData = function(field, key, text) {
        $scope.customFieldData[field.field_name] = {
            value: key,
            text: text
        };
    };
    /*-- Reset Location --*/
    $scope.resetLocation = function() {
      $scope.location = {};
      $scope.adss.load();
    };
    /*-- Reset Category --*/
    $scope.resetCategory = function() {
      $scope.category = {};
      $scope.adss.load();
    };

    /*
     * function that handles the back button
     */
    $scope.backButtonClicked = function() {
      var form = $scope.vm.addForm;

      if(form.$pristine || $scope.saved) {
          return navigationService.go('app.list');
      }

      var exitPopup = $ionicPopup.show({
        template: 'تغییرات شما هنوز ذخیره  نشده است. آیا مایلید که از این صفحه خارج شوید؟',
        cssClass: 'no-header-popup loading-popup',
        scope: $scope,
        buttons: [
          {
            text: '<b>بلی</b>',
            type: 'button-balanced',
            onTap: function() {
              exitPage();
            }
          },
          {
            text: '<b>خیر</b>',
            type: 'button-assertive',
            onTap: function() {
              exitPopup.close();
            }
          }
        ]
      });

      /*
       * internal Function for exiting the add ads page
       */
      var exitPage = function() {
          $scope.formData = {};
          $scope.imageData = null;
          $scope.saved = false;
          form.$setPristine();
          $scope.submitted = false;
          exitPopup.close();
          navigationService.go('app.list');
      };
    };

    /*
     * Redirects back
     */
    $scope.goBack = function() {
        navigationService.go('app.my-ads');
    };

    /**
     * Resets the images Array
     */
    $scope.setImagesArray = function() {
        if(typeof $scope.formData.images == 'undefined') {
            $scope.formData.images = [];
        }

        if($scope.imagesCount > 0) {
            for(var j = 1; j <= $scope.imagesCount; j++) {
                $scope.formData.images.push({
                    id: j,
                    isLoading: false,
                    isUploaded: false,
                    isFailed: false,
                    data: null,
                    uploaded_name: null,
                    isPreUploaded: false
                });
            }
        }


        if(typeof $scope.images != 'undefined' && $scope.images.length > 0) {
            angular.forEach($scope.images, function(image, index) {
                if( ! image.endsWith('default.jpg') ) {
                    var found_image = _.find($scope.formData.images, {id: (index+1)});
                    found_image.data = image;
                    found_image.isPreUploaded = true;
                }
            });
        }
    };

    /**
     * Callback for when plan is changed
     * @param new_plan
     */
    $scope.planChanged = function(new_plan) {
        $scope.imagesCount = new_plan.MAX_IMG_COUNT;
        $scope.hasLink = false;
        if(new_plan.HAS_LINK == 2) {
            $scope.hasLink = true;
        }
        $scope.setImagesArray();
    };

    $scope.expandText = function(input_id){
        var element = document.getElementById(input_id);
        var currentHeight = element.scrollHeight;
        if(currentHeight > 50) {
            element.style.height = currentHeight + "px";
        }
    };
}]);


