angular.module('koClassified.services.adsService', [])

  .service('adsService', ['$q', '$localStorage', 'lodash', 'adsFactory', function ($q, $localStorage, _, adsFactory) {

    var self = this;

    self.locations = [];
    self.categories = [];
    self.isLoadingLocations = false;
    self.isLoadingCategories = false;
    self.isLoadingMore = false;
    self.isLoadingSettings = false;
    self.settings = [];
    self.isLoadingPlans = false;
    self.plans = [];
    self.isVisitCounted = false;
    self.defaultPlan = [];
    self.is_loading_custom_values = false;

    self.reset = function() {
        self.items     = [];
        self.hasMore   = true;
        self.isLoading = false;
        self.params.page = 1;
    };

    self.params = {
        location : null,
        category : null,
        keyword : null,
        type: null,
        page : 1,
        counted : self.isVisitCounted
    };

    /*
    * posts the newly submitted ads to server
    */
    self.post = function(data) {
        var defered = $q.defer();

        adsFactory.postAds(data).then(function(result) {
            defered.resolve(result);
        }, function(err) {
            defered.reject(err);
        });

        return defered.promise;
    };

    /*
     * posts the newly submitted ads to server
     */
    self.update = function(data) {
      var defered = $q.defer();

      adsFactory.updateAds(data).then(function(result) {
          defered.resolve(result);
      }, function(err) {
          defered.reject(err);
      });

      return defered.promise;
    };

    /*
     * loads the ads items from Factory
     */
    self.load = function () {
        var defered = $q.defer();
        self.isLoading = true;
        self.params.page = 1;
        self.hasMore = true;

        adsFactory.getAds(self.params).then(function(data) {
            if(data.length === 0) {
                self.hasMore = false;
            }

            self.isVisitCounted = true;
            self.params.counted = true;

            self.items = data;
            //self.isLoading = false;
            defered.resolve();
        });

        return defered.promise;
    };

    /*
    * Load Locations from adsFactory
    */
    self.loadLocations = function () {
        var defered = $q.defer();

        self.isLoadingLocations = true;
        self.locations = adsFactory.getLocations().then(function(locations) {
            self.locations = locations;
            self.isLoadingLocations = false;
            defered.resolve();
        });

        return defered.promise;
    };

    /*
     * Load Locations from adsFactory
     */
    self.loadSettings = function () {
        var defered = $q.defer();

        if (! _.isEmpty(self.settings)) {
            defered.resolve(self.settings);
            defered.resolve(self.settings);
        }

        //if(self.isLoadingSettings) {
        //    console.log('stop 2');
        //    return false;
        //}

        self.isLoadingSettings = true;

        if (_.isEmpty(self.settings)) {
            adsFactory.getSettings().then(function (settings) {
                self.settings = settings;
                self.isLoadingSettings = false;

                defered.resolve(self.settings);
            });
        }

        return defered.promise;
    };

    /*
     * Load default plan
     */
    self.loadDefaultPlan = function () {
        var defered = $q.defer();

        if(self.defaultPlan.length > 0) {
            defered.resolve(self.defaultPlan);
        }

        self.isLoadingDefaultPlan = true;
        adsFactory.getDefaultPlan().then(function(plan) {
            self.defaultPlan = plan;
            self.isLoadingDefaultPlan = false;
            defered.resolve(self.defaultPlan);
        });

        return defered.promise;
    };

    /*
     * Load Locations from adsFactory
     */
    self.loadPlans = function () {
        var defered = $q.defer();

        if(self.plans.length > 0) {
            defered.resolve(self.plans);
        }

        self.isLoadingPlans = true;
        adsFactory.getPlans().then(function(plans) {
            self.plans = plans;
            self.isLoadingPlans = false;
            defered.resolve(self.plans);
        });
        return defered.promise;
    };

    /*
    * Load Categories from adsFactory
    */
    self.loadCategories = function () {
        var defered = $q.defer();

        self.isLoadingCategories = true;
        self.categories = adsFactory.getCategories().then(function(categories) {
            self.isLoadingCategories = false;
            self.categories = categories;
            defered.resolve();
        });

        return defered.promise;
    };

    /*
    * Refreshes the list of items
    */
    self.refresh = function () {
        var defered = $q.defer();
            self.reset();
            self.load().then(function() {
                self.isLoading = false;
            });
            defered.resolve();
        return defered.promise;
    };

    /*
    * Loads the data for next page in the ads list
    */
    self.nextPage = function() {
        var defered = $q.defer();
        self.params.page += 1;

        self.isLoadingMore = true;
        adsFactory.getAds(self.params).then(function(data)
        {
             if(data.length === 0) {
                 self.hasMore = false;
             }

            self.isVisitCounted = true;
            self.params.counted = true;

            self.items = self.items.concat(data);

            self.isLoadingMore = false;
            defered.resolve();
        });

        return defered.promise;
    };

    /*
    * Returns all ads of the user with given token
    */
    self.allAdsOfUser = function(token) {
        var defered = $q.defer();

        adsFactory.allAdsOfUser(token).then(function (result) {
            if(result.status) {
                self.myAdsItems = result;
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    self.resetMyAds = function() {
        self.myAdsItems = {};
    };

    self.allFavourites = function() {
        if($localStorage.favourites) {
            return $localStorage.favourites;
        }
        return false;
    };

    /*
    * toggles the ads in favourites list
    */
    self.toggleFavourite = function(ads) {
        if(! $localStorage.favourites) {
            $localStorage.favourites = [];
        }

        if(! self.isFavourited(ads.ID)) {
            $localStorage.favourites.push(ads);
            return true;
        } else {
            self.removeFromFavourites(ads);
            return false;
        }
    };

    /*
    * checks whether the ads with the given id is in favourites list or not
    */
    self.isFavourited = function(ads_id) {
        var ads_record = self.loadFromFavourites(ads_id);
        return !!ads_record;
    };

    /*
    * Removes the ads from favourites list
    */
    self.removeFromFavourites = function(ads) {
        _.remove($localStorage.favourites, function(currentObject) {
            return currentObject.ID == ads.ID;
        });
    };

    /*
    * Returns the total count of favourites
    */
    self.favouritesCount = function() {
        if($localStorage.favourites) {
            return $localStorage.favourites.length;
        }
        return 0;
    };

    /*
    * Returns the url to the featured image of the given Item
    */
    self.showFeaturedImage = function(item, remove_cache) {
        remove_cache = remove_cache || false;

        if(!item) {
            return false;
        }
        if(self.isTextAds(item)) {
            return 'img/text-ads.jpg';
        }

        if (remove_cache) {
            return item.featured_image + '?rand=' + remove_cache;
        }

        return item.featured_image;
    };

    /*
    * Returns the Url of featured image for the specific item
    *
    * @image array|string : the image variable of the desired ads item
    */
    self.images = function (image) {
        try {
          return JSON.parse(image);
        } catch (e) {
          return [image];
        }
    };

    /*
    * Returns the full location of the given ads
    *
    * @item object : the object of the given ads item
    */
    self.location = function (item) {
        var state    = item.state || '';
        var city  = item.city || '';
        var location = '';
        if (state) {
          location += state.STATE_NAME + '، ';
        }
        if (city) {
          location += city.CITY_NAME;
        }

        return (location) ? location : ' -- ';
    };

    /*
    * Returns the full html version of subcategory and it's parent(if exists) for the given Ads item
    *
    * @item | Ads object
    */
    self.fullSubcategory = function(item) {
        var output = '';
        if(item.branch.parent){
            output += item.branch.parent.SUBCATEGORY_NAME + '، ';
        }
        output += item.branch.SUBCATEGORY_NAME;

        return output;
    };

    self.isTextAds = function(item) {
      return (item.special.plan.IS_TEXT == 2);
    };
    self.isSpecialAds = function(item) {
       return (item.special.plan.PLAN_STARS > 0);
    };
    self.isShown = function(item) {
        return item.special.IS_SHOWN == 2;
    };
    self.isActive = function(item) {
        return item.IS_ACTIVE == 2;
    };
    self.isSuspended = function(item) {
        return !!(! self.isShown(item) && ! self.isActive(item));
    };
    self.isPayed = function(item) {
        return item.special.IS_PAID == 2;
    };
    self.isWaitingForPayment = function(item) {
        return !!(!self.isFreeAds(item) && !self.isPayed(item));
    };
    self.isEditable = function(item) {
        return !self.isExpired(item);
    };
    self.isExpired = function(item) {
      return item.special.IS_EXPIRED == 2;
    };


    /*
    * Returns an ads by it's id
    *
    * @id string | integer : id of the desired ads item
    */
    self.findWithId = function (ads_id) {
        var defered = $q.defer();
        self.isLoading = false;

        // if already in the loaded list, return it
        var ads = _.find(self.items, {'ID': parseInt(ads_id)});
        if(ads) {
            adsFactory.incrementViews(ads_id);
            defered.resolve(ads);
        }

        // if already cached in the favourites list, load from cache
        else if(self.isFavourited(ads_id)) {
            ads = self.loadFromFavourites(ads_id);
            defered.resolve(ads);
        }

        // otherwise load it via http call
        else {
            self.isLoading = true;
            adsFactory.findById(ads_id).then(function(result) {
                adsFactory.incrementViews(ads_id);
                self.isLoading = false;
                defered.resolve(result);
            }, function(err) {
                self.isLoading = false;
                console.log('failed from service');
                defered.reject(err);
            });
        }

        return defered.promise;
    };


    /*
     * Returns an ads by it's id
     *
     * @id string | integer : id of the desired ads item
     */
    self.findWithIdForEdit = function (ads_id) {
        var defered = $q.defer();
        self.isLoading = false;

        self.isLoading = true;
        adsFactory.findByIdForEdit(ads_id).then(function(result) {
            self.isLoading = false;
            defered.resolve(result);
        }, function(err) {
            self.isLoading = false;
            console.log('failed from service');
            defered.reject(err);
        });

        return defered.promise;
    };

    /*
    * Returns all cities related to a specific state from the list of locations
    *
    * @state | object : object the given state
    */
    self.cities = function (state, showNeutralOption) {
        var cities = _.filter(self.locations, {'STATE_ID': state.ID});

        showNeutralOption = showNeutralOption || 'yes';
        if(showNeutralOption == 'yes') {
            cities.unshift({'CITY_NAME': ' همه شهرهای ' + state.STATE_NAME, 'ID': 'STATE_'+state.ID, 'STATE_ID': state.ID, 'filter_name': state.STATE_NAME, is_all_selector: true});
        }

        return cities;
    };

    /*
    * Returns all parent categories from the list of categories
    */
    self.parentCategories = function () {
        return _.filter(self.categories, {'PARENT': null});
        //return _.filter(self.categories, {'PARENT': ""});
    };

    /*
    * Returns all subsidiaries related to the given subcategory
    *
    * @subcategory | object : object of the parent subcategory
    */
    self.subsidiaries     = function (subcategory, showNeutralOption) {
        var subsidiaries = _.filter(self.categories, {'CATEGORY_ID': subcategory.CATEGORY_ID, 'PARENT': subcategory.ID});

        showNeutralOption = showNeutralOption || 'yes';
        if(subsidiaries.length > 0 && showNeutralOption == 'yes') {
            subsidiaries.unshift({CATEGORY_ID: subcategory.CATEGORY_ID, ID: 'SUBCATEGORY_'+subcategory.ID, PARENT: 0, SUBCATEGORY_NAME: 'تمام زیردسته های ' + subcategory.SUBCATEGORY_NAME, filter_name: subcategory.SUBCATEGORY_NAME, is_all_selector: true});
        }

        return subsidiaries;
    };

    self.isRejected = function(item) {
        if(item.status_code == 'rejected') {
            return true;
        }

        return false;
    };



    /*
     * Load the custom field and values for the given ads
     */
    self.loadCustomValues = function(ads) {
        var defered = $q.defer();

        if(!ads.custom_fields) {
            self.is_loading_custom_values = true;
            adsFactory.getCustomValues(ads.ID).then(function(custom_fields) {
                ads.custom_fields = custom_fields;
                self.is_loading_custom_values = false;
                defered.resolve(custom_fields);
            }, function(error) {
                defered.reject('error for custom fields in service');
            });
        } else {
            defered.resolve(ads.custom_fields);
        }

        return defered.promise;
    };

    /*
     * Load the custom field and values for the given ads
     */
    self.loadCustomFields = function(category_id, subcategory_id, subsidiary_id) {
        var defered = $q.defer();

        adsFactory.getCustomFields(category_id, subcategory_id, subsidiary_id).then(function(custom_fields) {
            defered.resolve(custom_fields);
        }, function(error) {
            defered.reject('error for custom fields in service');
        });

        return defered.promise;
    };

    /*
     * Returns all radio button fields from the given custom fields list
     */
    self.getRadioBoxes = function(fields) {
        return _.filter(fields, {'type' : 'radio'});
    };

    /**
     * Checks whether the registration needs activation or not
     *
     * @return boolean
     */
    self.registerNeedsActivation = function () {
        return (self.settings.REGISTER_NEEDS_ACTIVATE === 'YES');
    };

    /**
     * Check if activation can be done via email
     *
     * @return boolean
     */
    self.canActivateWithEmail = function() {
        var activation_method = self.settings.REGISTER_VERIFICATION_METHOD;
        return (self.registerNeedsActivation() && ( activation_method === 'EMAIL' || activation_method === 'ALL') );
    };

    /**
     * Check if activation is only possible via email
     *
     * @return boolean
     */
    self.isActivateWithOnlyEmail = function() {
        var activation_method = self.settings.REGISTER_VERIFICATION_METHOD;
        return (self.registerNeedsActivation() && activation_method === 'EMAIL');
    };

    /**
     * Check if activation can be done via SMS
     *
     * @return boolean
     */
    self.canActivateWithSMS = function() {
        var activation_method = self.settings.REGISTER_VERIFICATION_METHOD;
        return (self.registerNeedsActivation() && ( activation_method === 'SMS' || activation_method === 'ALL') );
    };

    /**
     * Check if activation is only possible via sms
     *
     * @return boolean
     */
    self.isActivateWithOnlySms = function() {
        var activation_method = self.settings.REGISTER_VERIFICATION_METHOD;
        return (self.registerNeedsActivation() && activation_method === 'SMS');
    };

    /**
     * Returns the title for username field based on the type of activation
     *
     * @return String
     */
    self.usernameTitle = function (is_for_reset_password_form) {
        is_for_reset_password_form = is_for_reset_password_form || false;

        if(self.usernameType() === 'email') {
          return 'ایمیل';
        }
        if(self.usernameType() === 'mobile') {
          return 'شماره موبایل';
        }
        if(self.usernameType() === 'all') {
            if(is_for_reset_password_form && ! self.registerNeedsActivation()) {
                return 'نام کاربری یا ایمیل';
            }
            return 'شماره موبایل یا ایمیل';
        }

        return null;
    };

    /**
     * Returns the type of username that can be entered
     *
     * @return String
     */
    self.usernameType = function () {

        if(self.isActivateWithOnlyEmail()) {
            return 'email';
        }
        if(self.isActivateWithOnlySms()) {
            return 'mobile';
        }

        return 'all';
    };

	  /**
     * sends request to deletes the given item
     *
     * @param item
     * @param token
     */
    self.deleteItem = function(item, token) {
        _.remove(self.items, {ADS_ID: item.ID});
        adsFactory.deleteItem(item.ID, token);
    };


      /*
       * Checks whether the given plan is a special plan or not
       */
      self.isSpecialPlan = function(plan) {
          //return plan.PLAN_PRICE !== '0';
           return plan.PLAN_PRICE !== 0;
      };

    /*
     * Returns all states from the list of locations
     */
    self.states = function () {
      //return _.filter(self.locations, { 'STATE_ID' : '0' });
      return _.filter(self.locations, { 'STATE_ID' : 0 });
    };

    /*
     * Returns all subcategories related to the given category
     *
     * @category | object : object of the parent category
     */
    self.subcategories    = function (category, showNeutralOption) {
        //var subcategories = _.filter(self.categories, {'CATEGORY_ID': category.ID, 'PARENT': '0'});
        var subcategories = _.filter(self.categories, {'CATEGORY_ID': category.ID, 'PARENT': 0});

        showNeutralOption = showNeutralOption || 'yes';
        if(showNeutralOption == 'yes') {
            subcategories.unshift({CATEGORY_ID: category.ID, ID: 'CATEGORY_'+category.ID, PARENT: 0, SUBCATEGORY_NAME: 'تمام زیردسته های ' + category.CATEGORY_NAME, filter_name: category.CATEGORY_NAME, is_all_selector: true});
        }

        return subcategories;
    };

    /*
    * Checks whether the ads has free plan or not
    */
    self.isFreeAds = function(item) {
        //return !!(item.special.plan.PLAN_STARS === '0' && item.special.plan.IS_TEXT == 1);
        return !!(item.special.plan.PLAN_STARS === 0 && item.special.plan.IS_TEXT == 1);
    };

    self.isFreePlan = function(plan) {
        //return !!(plan.PLAN_STARS === '0' && plan.IS_TEXT == 1);
        return !!(plan.PLAN_STARS === 0 && plan.IS_TEXT == 1);
    };

    /*
     * Returns the ads with given id from favourites list
     */
    self.loadFromFavourites = function(ads_id) {
        //return _.find($localStorage.favourites, {ID: ads_id});
        return _.find($localStorage.favourites, {ID: parseInt(ads_id)});
    };


    /* functions to run on Instantiation */
    self.reset();
  }]);
