angular.module('koClassified.services.auth', [])

.service('authService', ['$q', '$localStorage', 'authFactory', 'adsService', function ($q, $localStorage, authFactory, adsService) {

    var self = this;

    self.logged = false;
    self.token = null;

    /**
     *  holds the type of auth :: normal, submit
     */
    self.authType = 'normal';

    /**
     * Holds the username entered by user for login part in submit ads
     */
    self.triedUsername = null;

    /**
     * Holds the action type of the page for entering password
     */
    self.passwordPageAction = null;

    /**
     * Holds the user activation method
     */
    self.activationMethod = null;

    /**
     * Holds the counter for resend user's activation token
     */
    self.activationResendTime = 0;

    /**
     * Holds Temporary login token for forgot password
     */
    self.temporaryToken = null;

    /*
     * Checks whether the user is logged in or not
     */
    self.isLoggedIn = function() {
        if(self.logged && self.token) {
            return true;
        }

        return !!self.getUser();
    };

    /*
    * handles the login functionality of a user
    */
    self.login = function (data)
    {
        var defered = $q.defer();

        authFactory.login(data).then(function (result)
        {
            if (result.status && result.token.length > 0) {
                self.successAuthCallback(result, data);
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /*
     * handles the user register functionality
     */
    self.register = function (data)
    {
        var defered = $q.defer();

        authFactory.register(data).then(function (result)
        {
            if (result.status && result.token.length > 0) {
                self.successAuthCallback(result, data);
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

       return defered.promise;
    };

    /*
     * handles the login functionality of a user
     */
    self.loginOrRegister = function (data)
    {
        var defered = $q.defer();

        authFactory.loginOrRegister(data).then(function (result)
        {
            defered.resolve(result);
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /*
     * handles the user register functionality
     */
    self.checkActivationCode = function (data)
    {
        var defered = $q.defer();

        authFactory.checkActivationCode(data).then(function (result) {
          if (result.status && result.token.length > 0) {
                self.successAuthCallback(result, data);
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /*
     * ReSends activation code to user
     */
    self.resendActivationCode = function (data)
    {
        var defered = $q.defer();

        if(self.resendActivationCode > 0) {
            defered.reject({ code: 'timer', 'message': 'لطفا تا زمان اتمام شمارشگر منتظر بمانید'});
        }

        authFactory.resendActivationCode(data).then(function (result) {
            if (result.status) {
                self.activationResendTime = result.resend_time;
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /*
     * handles the login functionality of a user
     */
    self.checkForgotParam = function (data)
    {
        var defered = $q.defer();

        authFactory.checkForgotParam(data).then(function (result) {
            defered.resolve(result);
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /**
     * Checks the verification code for auth forgot password
     */
    self.checkForgotVerificationCode = function (data)
    {
        var defered = $q.defer();

        authFactory.checkForgotVerificationCode(data).then(function (result) {
            if (result.status && result.token.length > 0) {
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /*
     * handles the reset Forgotten Password
     */
    self.resetForgottenPassword = function (data)
    {
      var defered = $q.defer();

      authFactory.resetForgottenPassword(data).then(function (result) {
        if (result.status) {
            defered.resolve(result);
        } else {
            defered.reject(result.errors);
        }
      }, function () {
          defered.reject('مشکلی در اجرای عملیات رخ داده است');
      });

      return defered.promise;
    };

    /*
     * handles the Forgot Password functionality
     */
    self.resetPassword = function (data)
    {
        var defered = $q.defer();

        authFactory.resetPassword(data).then(function (result)
        {
            if (result.status) {
                defered.resolve('SUCCESS');
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };


    /*
     * handles the change Password functionality
     */
    self.changePassword = function (data)
    {
        var defered = $q.defer();

        data.token = self.token;
        authFactory.changePassword(data).then(function (result)
        {
            if (result.status) {
                defered.resolve('SUCCESS');
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('مشکلی در اجرای عملیات رخ داده است');
        });

        return defered.promise;
    };

    /*
    * Returns the ads count of the logged In user
    */
    self.adsCount = function() {
        var defered = $q.defer();

        authFactory.adsCount(self.token).then(function (result) {
            if(result.status) {
                defered.resolve(result);
            } else {
                defered.reject(result.errors);
            }
        }, function () {
            defered.reject('ERROR');
        });

        return defered.promise;
    };

    /*
     * Handles the logout functionality of user
     */
    self.logout = function() {
        self.logged = false;
        self.token = null;
        delete $localStorage.user;
        delete $localStorage.token;
        delete $localStorage.email;
        delete $localStorage.mobile;
    };

    /*
     * Function to be called after successful  login and register
     */
    self.successAuthCallback = function(result, data) {
        self.logged = true;
        self.saveToken(result.token);
        self.saveEmail(result.email);
        self.saveMobile(result.mobile);
        self.saveUser(data.username);

        self.triedUsername = null;
        self.passwordPageAction = null;
        self.activationMethod = null;
        self.activationResendTime = 0;
    };

    /*
    * Saves the user in local storage
    */
    self.saveUser = function(data) {
        $localStorage.user = data;
    };

    /*
    * Retrieves the user from local storage
    */
    self.getUser = function() {
        if($localStorage.user) {
            return $localStorage.user;
        }

        return null;
    };

    /*
    * Saves the email in local storage for further access
    */
    self.saveEmail = function(email) {
        $localStorage.email = email;
    };

    /*
     * Saves the email in local storage for further access
     */
    self.saveMobile = function(mobile) {
        $localStorage.mobile = mobile;
    };

    /*
    * Retrieves the email from local storage
    */
    self.getEmail = function(useStoredEmail) {
        useStoredEmail = useStoredEmail || 'yes';

        if(self.getUser()) {
            return $localStorage.user;
        }

        if(useStoredEmail === 'yes' && $localStorage.email) {
            return $localStorage.email;
        }

        return null;
    };

    /**
     *  Returns saved user email
     */
    self.getUserEmail = function() {
        if($localStorage.email) {
            return $localStorage.email;
        }

        return null;
    };

    /**
     *  Returns saved user mobile
     */
    self.getUserMobile = function() {
        if($localStorage.mobile) {
            return $localStorage.mobile;
        }
        return null;
    };

    self.saveToken = function(token) {
        self.token = token;
        $localStorage.token = token;
    };

    self.getToken = function() {
        if($localStorage.token) {
            return $localStorage.token;
        }

        return null;
    };

    /**
     * Checks whether the user's token has been expired or not
     */
    self.checkTokenExpiry = function() {
        if(self.isLoggedIn() && self.token) {
              authFactory.checkTokenExpiry(self.token).then(function(response){
                  if( ! response.status ) {
                      self.logout();
                  }
              });
          }
    };

     /**
     * Returns the username that user tried for login in submit ads part
     *
     * @returns string|null
     */
    self.getTriedUsername = function() {
        return self.triedUsername;
    };

    /**
     * checks if current auth page is for submit or not
     *
     * @returns {boolean}
     */
    self.isSubmitAuth = function() {
        return (self.authType === 'submit');
    };

    /**
     * checks if current auth page is a normal auth or not
     *
     * @returns {boolean}
     */
    self.isNormalAuth = function() {
        return (self.authType === 'normal');
    };

    self.token = self.getToken();
    self.logged = self.isLoggedIn();

}]);
