angular.module('koClassified.services.connectivityMonitor', [])

.service('connectivityMonitor', ['$rootScope', '$cordovaNetwork', '$ionicPopup', '$timeout', '$state',
                        function( $rootScope,   $cordovaNetwork,   $ionicPopup,   $timeout,   $state)
{
    var self = this;

    self.localScope = null;

    /*
    * Sets the local scope for service
    */
    self.setScope = function(localScope) {
        self.localScope = localScope;
    };

    /*
    * check whether the internet connection is available
    */
    self.isOnline = function(){
        if(ionic.Platform.isWebView()){
            return $cordovaNetwork.isOnline();
        } else {
            return navigator.onLine;
        }
    };

    /*
    * check whether the internet connection is NOT available
    */
    self.isOffline = function(){
        if(ionic.Platform.isWebView()){
            return !$cordovaNetwork.isOnline();
        } else {
            return !navigator.onLine;
        }
    };

    /*
    * watch for changing in connection status
    */
    self.startWatching = function(){
        if(ionic.Platform.isWebView())
        {
            $rootScope.$on('$cordovaNetwork:online', function(event, networkState){
                self.localScope.onNetworkAvailable();
            });

            $rootScope.$on('$cordovaNetwork:offline', function(event, networkState){
                $rootScope.showPopup();
            });

        } else {
            window.addEventListener("online", function(e) {
                alert("went online");
            }, false);

            window.addEventListener("offline", function(e) {
                alert("went offline");
            }, false);
        }
    };

    /*
    * Shows the popup when can't reaching internet connection
    */
    self.showPopup = function() {
        if($rootScope.connectionPopup) {
            $rootScope.connectionPopup.close();
        }

        $rootScope.connectionPopup = $ionicPopup.show({
            templateUrl : 'templates/popups/connection_status.html',
            scope : $rootScope,
            cssClass : 'no-header-popup loading-popup ' + $rootScope.main_theme
        }, 1000);
    };

   /*
   * Retries the connection check
   */
    $rootScope.retryConnection = function() {
        $rootScope.connectionPopup.close();

        if(self.isOffline()) {
            $timeout(function() {
                self.showPopup();
            });
            return false;
        }

        // if internet connection was mode
        return self.localScope.onNetworkAvailable();
    };

    /*
    * Dismiss the connection popup
    */
    $rootScope.cancelConnection = function() {
        $rootScope.connectionPopup.close();
    };

   /*
   * Redirect to favourites page
   */
    $rootScope.goToFavourites = function() {
        $rootScope.cancelConnection();
        $state.go('app.favourites');
    };

}]);
