angular.module('koClassified.services.modalService', [])
  .service('modalService', ['$ionicModal', '$q', '$rootScope', 'adsService', 'connectivityMonitor', function ($ionicModal, $q, $rootScope, adsService, connectivityMonitor) {

    var self = this;
    var modalScope = $rootScope;
    modalScope.states = [];
    modalScope.categories = [];
    modalScope.plans = [];

    self.controllerScope = null;
    self.currentSelection = null;

    /* assigning adsService to rootScope so it can be accessible within modals */
    modalScope.root = {};
    modalScope.root.adsService = adsService;

    self.init = function(controllerScope)
    {
        self.controllerScope = controllerScope;
        /*---------------------------------------------------------
                            Location Modals
        ----------------------------------------------------------*/
        // load cities of the given state from adsService
        modalScope.loadCities = function (state) {
            modalScope.cities = adsService.cities(state, modalScope.showNeutralOption);

            modalScope.hideModal('states');
            modalScope.showModal('cities', state);
        };

        // call setLocation function from the controller to do specified job after setting a location
        modalScope.setLocation = function(state) {
            controllerScope.setLocation(state);
            modalScope.hideModal('states');
            modalScope.hideModal('cities');
        };

        // call resetLocation function from the controller to do specified job after reseting the location
        modalScope.resetLocation = function() {
            modalScope.hideModal('states');
            controllerScope.resetLocation();
        };

      /*---------------------------------------------------------
                          Category Modals
       ----------------------------------------------------------*/
        // load subcategories of the given category from adsService
        modalScope.loadSubcategories = function(category) {
            modalScope.subcategories = adsService.subcategories(category, modalScope.showNeutralOption);
            controllerScope.setSpecificCategory(category);

            modalScope.hideModal('categories');
            modalScope.showModal('subcategories');
        };

        // load subsidiaries of the given subcategory from adsService if exists
        modalScope.loadSubsidiaries = function(subcategory) {
            modalScope.subsidiaries = adsService.subsidiaries(subcategory, modalScope.showNeutralOption);
            controllerScope.setSpecificSubCategory(subcategory);

            if(modalScope.subsidiaries.length > 0) {
                modalScope.hideModal('categories');
                modalScope.hideModal('subcategories');
                modalScope.showModal('subsidiaries');
            } else {
                modalScope.setCategory(subcategory);
            }
        };

        // call setCategory function from the controller to do specified job after setting the category
        modalScope.setCategory = function(category) {
            controllerScope.setCategory(category);

            modalScope.hideModal('categories');
            modalScope.hideModal('subcategories');
            modalScope.hideModal('subsidiaries');
        };

        // call resetCategory function from the controller to do specified job after reseting the category
        modalScope.resetCategory = function() {
          modalScope.hideModal('categories');
          controllerScope.resetCategory();
        };

        // sets the selected plan
        modalScope.setPlan = function(plan) {
            controllerScope.setPlan(plan);
            modalScope.hideModal('plans');
        };

        modalScope.setCustomFieldModalData = function(field, key, text) {
            controllerScope.setCustomFieldModalData(field, key, text);
            modalScope.hideModal('dynamic');
        };
    };

    /* load locations */
    modalScope.loadLocations = function(data) {
        $rootScope.showNeutralOption = 'yes';
        if(data === 'no') {
          $rootScope.showNeutralOption = 'no';
        }
        if(adsService.locations.length === 0 || modalScope.states.length === 0) {
            adsService.loadLocations().then(function () {
                modalScope.states = adsService.states();
            });
        }
    };

    /* load locations */
    modalScope.loadCategories = function(data) {
        $rootScope.showNeutralOption = 'yes';
        if(data === 'no') {
          $rootScope.showNeutralOption = 'no';
        }

        if(adsService.categories.length === 0 || modalScope.categories.length === 0) {
            adsService.loadCategories().then(function () {
                modalScope.categories = adsService.parentCategories();
            });
        }
    };

    modalScope.loadPlans = function() {
        if(adsService.plans.length === 0 || modalScope.plans.length === 0) {
            adsService.loadPlans().then(function (plans) {
                modalScope.plans = plans;
            });
        }
    };

    //modalScope.loadLocations();
    //modalScope.loadCategories();


    $ionicModal.fromTemplateUrl('templates/modals/locations.html', {
      scope    : modalScope,
      animation: 'slide-in-up'
    }).then(function (modal) {
      modalScope.loadLocations();
      modalScope.statesModal = modal;
    });

    $ionicModal.fromTemplateUrl('templates/modals/cities.html', {
      scope    : modalScope,
      animation: 'slide-in-up'
    }).then(function (modal) {
      modalScope.citiesModal = modal;
    });




    $ionicModal.fromTemplateUrl('templates/modals/categories.html', {
      scope      : modalScope,
      "animation": "slide-in-up"
    }).then(function (modal) {
      modalScope.loadCategories();
      modalScope.categoriesModal = modal;
    });

    $ionicModal.fromTemplateUrl('templates/modals/dynamic.html', {
      scope      : modalScope,
      "animation": "slide-in-up"
    }).then(function (modal) {
      modalScope.dynamicModal = modal;
    });

    $ionicModal.fromTemplateUrl('templates/modals/subcategories.html', {
      scope      : modalScope,
      "animation": "slide-in-up"
    }).then(function (modal) {
      modalScope.subcategoriesModal = modal;
    });

    $ionicModal.fromTemplateUrl('templates/modals/subsidiaries.html', {
      scope      : modalScope,
      "animation": "slide-in-up"
    }).then(function (modal) {
      modalScope.subsidiariesModal = modal;
    });


    /*
    * Creates Plan Selector Modal
    */
    $ionicModal.fromTemplateUrl('templates/modals/plans.html', {
        scope      : modalScope,
        "animation": "slide-in-up"
    }).then(function (modal) {
        modalScope.plansModal = modal;
    });

    /*
     * functions for show, hide and removing modals
     */
    modalScope.showModal   = function (modal_name, data, is_dynamic) {

        data = data || false;
        is_dynamic = is_dynamic || false;

        if(modal_name == 'categories') {
            modalScope.loadCategories(data);
            if(connectivityMonitor.isOffline()) {
              return connectivityMonitor.showPopup();
            }
        }

        if(modal_name == 'states') {
            modalScope.loadLocations(data);
            if(connectivityMonitor.isOffline()) {
                return connectivityMonitor.showPopup();
            }
        }

        if(modal_name == 'plans') {
            modalScope.showFreePlan = true;
            modalScope.currentPlanId = 0;

            if(typeof data != 'undefined' && data.type == 'no-free-plan') {
                modalScope.showFreePlan = false;
                modalScope.currentPlanId = data.current_plan_id;
            }
            modalScope.loadPlans();
            if(connectivityMonitor.isOffline()) {
                return connectivityMonitor.showPopup();
            }
        }

        //if(modal_name == 'rooms' || modal_name == 'brand' || modal_name == 'year') {
        if(is_dynamic) {
            modalScope.customFieldModalData = data;
            modal_name = 'dynamic';
        }

        modal_name = modal_name + 'Modal';
        modalScope.$eval(modal_name).show();

        // function to be called on controller when showing the modal
        var controllerFunctionName = modal_name + 'OnShow()';
        self.currentSelection = data;
        self.controllerScope.$eval(controllerFunctionName);
    };
    modalScope.hideModal = function (modal_name) {
      modal_name = modal_name + 'Modal';
      modalScope.$eval(modal_name).hide();
    };
    modalScope.removeModal = function (modal_name) {
      modal_name = modal_name + 'Modal';
      modalScope.$eval(modal_name).remove();
    };

    modalScope.$on('$destroy', function () {
      modalScope.removeModal('states');
      modalScope.removeModal('cities');
      modalScope.removeModal('categories');
      modalScope.removeModal('subcategories');
      modalScope.removeModal('subsidiaries');
      modalScope.removeModal('filters');
    });
  }]);
